; Oster-Contest 2013
;
; servo1: 1 ms-2 ms / 50Hz (futaba 0,6-2,4ms)
;
; Das Poti/ADC(2) wird per Interrupt in r16 geholt. 
; Das verbraucht etwa 20% der Gesammtzeit. 
; Dadurch sind die Zeiten nur ungefhr berechenbar. 
; 
; Mit PB.1=1 wird r18 einmal durchgezhlt (0-255), um den Pulsanfang von 0,8ms zu erzeugen, 
; dann wird r18 noch einmal durchgezhlt und mit r16(adc) verglichen, 
; bei bereinstimmung wird PB.1 auf 0 gesetzt, 
; nun sind 2,3ms vergangen und die Rest-Pause (17,7ms) beginnt. 
; 
; das Ganze funktioniert nur bei 1,2MHz
;
; Lock FF, Fuse 6A = 0b0110_1010, Fusehigh FF
;                  devide=^ ^^^^=9,6MHz 
;
;                      ATMEL ATtiny13 
;                      _____   _____
;                     |     |_|     |
;                   o-|Res       Vcc|-------o +5V
;                     |             |    
;                     |             | 
;                     |          sck|  
;                   o-|PB3       PB2|-o
;                     |ADC3     ADC1|  
;                     |             |     |-o Servo schwarz
;              ___    |         miso|   +5V o Servo rot
;           +-|___|---|PB4       PB1|-------o Servo gelb/braun
;           |  1k     |ADC2     oc0b|  
;           |         |             |  
;          _v_        |         mosi|  
;   +5V o-|___|-----+-|Gnd       PB0|-o
;          10k      | |         oc0a|  
;         Poti   |--+ |_____________|  
;
;
.DEVICE ATtiny13		;fr gavrasm, fr Symbolische Registerbezeichnungen (z.B.'DDRB')
.cseg				;codesegment
.org 0				;adr 0
				;Vector-Tabelle
	rjmp init		;Reset vector
	reti 			;Int0 interrupt vector
	reti	 		;PCINT0 vector
	reti 			;TC0 overflow vector
	reti			;Eeprom ready vector
	reti			;Analog comparator int vector
	reti			;TC0 CompA vector
	reti			;TC0 CompB vector
	reti			;WDT vector
	rjmp int_adc		;ADC conversion complete vector
;--------------------------------
Int_adc:			;Aufruf NUR, wenn neuer Wert verfgbar
	in r16,ADCh		;Lese ADC-Wert, obere 8Bit
	reti			;zurck ins Hauptprogramm
;--------------------------------
init:				;Vorbereitungen
	ldi r16,ramend		;Stack vorbereiten
	out spl,r16		;auf RAMEND setzen
	sbi ddrb,1		;Ausgang definieren
init_adc:			;
	ldi r16,0b00100010	;  - 0 1 -  - - 1 0
	Out Admux , R16		;ref-^ ^adlar   ^+^-adc2
	ldi r16,0b00000000	;  - - - -  - 0 0 0
	Out Adcsrb , R16	;             ^+^+^-Free Run
	ldi r16,0b11101111	;    1 1 1 0  1 1 1 1
	Out Adcsra , R16	;ena-^ ^-start  ^+^+^-prescale
	sei			;Set Enable Interrupt (-Flag)
;--------------------------------
main:			;Hauptprogramm
	sbi portb,1	;Pulsanfang			(1Cy) nur einmal
main0:			;Offset				(us ~ 256 * 3Cy) ungefhr 768us
	inc r18		;0-255-0			(1Cy)
	brbc 1,main0	;springe wenn Z=0		(2Cy)
;------------------------				
main1:			;+ adc * 6Cyclen		(us ~ 256 * 6Cy) ungefhr 1536us
	cpse r16,r18	;skip if R18=adc ?		(1Cy)
	rjmp main2	;nein, weiterzhlen		(2Cy)
	cbi portb,1	;ja, Pulsende			(1Cy) nur einmal
main2:			;zhlen
	inc r18		;0-255-0			(1Cy)
	brbc 1,main1	;springe wenn Z=0		(2Cy)
;------------------------ 				
	ldi r25,79	;pausenlnge (256-79)*256/8 	(1Cy) nur einmal
main3:			;17,7 ms			(us ~ (256-79) * 100) ungefhr 17700us
	adiw r24,8	;step 8				(2Cy)
	brbc 1,main3	;springe wenn Z=0		(2Cy)
	rjmp main	;nochmal			(2Cy) nur einmal
;end
